#include "LCDv1.h"
#include "MC74HC595.h"

#define LCD_BackLight_BIT          (1<<1)
#define LCD_RS_BIT                 (1<<2)
#define LCD_E_BIT                  (1<<3)
#define LCD_Data_Mask               0xF0
#define LCD_CMD_Mask                0x0E

unsigned char SPI_DAT=0;

void LCD_init(void)
{
        Delay_ms(17);
        LCD_Write(0x30);
        Delay_ms(1);
        LCD_Write(0x30);
        Delay_ms(1);
        LCD_Write(0x30);
        Delay_ms(1);
        LCD_Write(0x20);  //4 bit wide bus
                  
        LCD_Send_Cmd(0x28);           //4bit wide bus 2 rows
        LCD_Send_Cmd(0x10);
        LCD_Send_Cmd(0x01);
        LCD_Send_Cmd(0x0F);

        LCD_Send_Cmd(LCD_CMD_Cursor_Off);
        LCD_Send_Cmd(LCD_CMD_Clear);
        Delay_ms(10);
}

void LCD_Write(u8 Dat)
{         
 SPI_DAT&=(LCD_RS_BIT|LCD_BackLight_BIT);
 SPI_DAT|=(Dat&LCD_Data_Mask)|LCD_E_BIT;
 MC74HC595_SendByte(SPI_DAT);
 Delay_us(5);
 SPI_DAT&=~LCD_E_BIT;
 MC74HC595_SendByte(SPI_DAT);
 Delay_us(5);
}
void LCD_Send_Cmd(u8 CMD)
{
        Delay_ms(5);
        SPI_DAT&=~LCD_RS_BIT;
        LCD_Write((CMD&0xF0));
        LCD_Write(((CMD&0x0F)<<4));
}

void lcd_set_position(u8 row, u8 colum)
{
        u8 position = 0x80;
        if (row > 0) { position |= 0x40;}
        LCD_Send_Cmd(position | colum);
}

void lcd_write_data(u8 dat)
{
        Delay_us(50);
        SPI_DAT|=LCD_RS_BIT;
        LCD_Write((dat&0xF0));
        LCD_Write(((dat&0x0F)<<4));
}

void lcd_print_char(u8 dat)
{
        if (dat == '\n')
        {LCD_Send_Cmd(0x80 | 0x40);} else {lcd_write_data(dat);}
}

void lcd_print_string(u8 * string)
{
        while (*string != 0x00){ lcd_print_char(*string++);}
}

void lcd_print_value(unsigned long val)
{
        u8 buffer[10] = {0};
        u8* head = buffer;
        u8 cnt=0;
        if (val!=0)
        {
                while( val )
                {
                    //*head++ = (val % 10)["0123456789"];
                    val /= 10;
                        cnt++;
                }
                while (cnt!=0){ cnt--; lcd_print_char(buffer[cnt]); }
        }
        else
        {
                lcd_print_char('0');
        }
}

void lcd_print_hex_value(long val)
{
        char buffer[10] = {0};
        char * head = buffer;
        char cnt=0;
        lcd_print_char('0');
        lcd_print_char('x');
        if (val<=9)        lcd_print_char('0');
        if (val!=0)
        {
                while( val )
                {
                    //*head++ = (val & 0x0F)["0123456789ABCDEF"];
                    val=val>>4;
                        cnt++;
                }
                while (cnt!=0){ cnt--; lcd_print_char(buffer[cnt]); }
        }
        else
        {
                lcd_print_char('0');
        }
        lcd_print_char(' ');
}

void LCD_BackLight (unsigned char State)
{
  if (State)  SPI_DAT|=LCD_BackLight_BIT;
  else   SPI_DAT&=~LCD_BackLight_BIT;
}